//----------------------------------------------------------------------------//
//=Purpose: Exercise 4.1 of Beyond Classic Designs & Analysis for Health      //
//          Data in Singapore (27 June to 8 July 2022)                        //
//
//=Data required: 
//  - Framingham_HDL.dta, 
//  - Hpylori_extract.dta
//
//=Stata packages/libraries required:
//  - meanscor: user-written packages using mean score method for missing 
//              covariate data in logistic regression models
//
//=Created by:
//  - Zihan 
//  - Francesca Graziano 
//  - Zilu Feng
//  - Zihan 
//  - Chuen Seng Tan
//  - Marie Reilly
//
//=Changes (if any):
//  -DDMMYYYY: Describe
//
//----------------------------------------------------------------------------//

//-Set the working directory to the data folder where the datasets are saved:
cd "D:/nbox/Education/Marie_short_course/Singapore course 2022/20220602/exercises/data"


////////////////////////////////////////////////////////////////////////////////
//                               Question 1      
//
//-1. In this exercise, you will sample a two-stage study from a cross-sectional 
//    of individuals from the Framingham cohort study, whose data are in the 
//    dataset Framingham_HDL.dta provided.  This data consists of 3,026 cohort 
//    members who had HDL and LDL measurements taken at their third visit. The 
//    outcome of interest is any CHD, with 777 cases and 2249 non-cases. Our 
//    objective is to estimate the effect of HDL and LDL on any CHD, adjusted 
//    for sex and hypertension. The variables are:
//
//    * randid (individual identification number)
//    * anychd (case indicator: 0=no CHD during follow-up, 1=CHD during follow-up)
//    * sex (1=male, 2=female)
//    * cursmoke (0=not current smoker, 1=current smoker)
//    * prevhyp (0=no prevalent hypertension, 1=prevalent hypertension)
//    * ldlc (low-density lipoprotein cholesterol, mg/dL) 
//    * hdlc (high-density lipoprotein cholesterol, mg/dL) 
//    * ldlc_cat (categorical LDL: desirable = <=100mg/dL, 
//                moderate = 101-159mg/dL, high = 160-189mg/dL, 
//                very high = >=190mg/dL)
//    * hdlc_cat (categorical HDL: low = <=40mg/dL, borderline = 41-59mg/dL, 
//                desirable = >=60mg/dL)
//    * hdlc_cat2 (hdlc_cat, releveled so ‘desirable’ is the reference) 
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//                               Question 1 (i)     
//
//-(i) Run a logistic regression on the full cross-sectional sample with anychd 
//     as outcome and covariates ldlc_cat, hdlc_cat2, sex, and prevhyp, to get 
//     adjusted ORs for HDL and LDL.
////////////////////////////////////////////////////////////////////////////////

use Framingham_HDL.dta,clear
logistic anychd i.ldlc_cat i.hdlc_cat2 i.sex i.prevhyp

////////////////////////////////////////////////////////////////////////////////
//                               Question 1 (ii)     
//
//-(ii) Now we will mimic a two-stage study where HDL and LDL were considered 
//      as ‘expensive’ covariates and were only measured for a second-stage 
//      subsample and were missing for the other individuals. To mimic a 
//      situation where the second-stage sample consists of all 777 cases and 
//      an equal number of controls, select/identify a subsample of 777 
//      controls (using a random number generator) and save a dataset 
//      containing all the cases and these controls (Note. You should set a 
//      “seed” so that you can reproduce the exact same data if needed).
////////////////////////////////////////////////////////////////////////////////

//set a seed using a random number:
set seed 1810

//generate shuffle:
gen u = runiform()
sort anychd u

//generate a variable to specify ppl selected in second-stage sample:
gen select = 0

//selects all (777) anychd==1 and 777 anychd==0:
by anychd: replace select = 1 if _n <= 777
drop if select==0

//save the dataset:
save subsample, replace


////////////////////////////////////////////////////////////////////////////////
//                               Question 1 (iii)     
//
//-(iii) For the dataset selected in (ii), create a weight variable that is 1 
//       for all cases and 2249/777 for each of the controls. Verify that 
//       running a weighted logistic regression (with robust SE) of just these 
//       (second-stage) data gives similar estimates to the full cohort results 
//       in (i).
////////////////////////////////////////////////////////////////////////////////

//weight for cases:
gen weight = 1 if anychd == 1

//weight for controls:
replace weight = 2249/777 if anychd == 0

//weighted logistic model:
logistic anychd i.ldlc_cat i.hdlc_cat2 i.sex i.prevhyp [pw = weight], vce(robust)

//this gives output which has similar estimates in the full cohort.


////////////////////////////////////////////////////////////////////////////////
//                               Question 1 (iv)     
//
//-(iv) Return to the full data set, and this time when you select the 777 
//      controls, do not delete the others from the data, but set their HDL and 
//      LDL variables to missing, and add a second-stage indicator variable to 
//      the data which is 0 for these missing observations and 1 for the cases 
//      and 777 controls. Repeat the logistic regression analysis this data set 
//      using the meanscor command in Stata or the survey package in R and 
//      compare your results to what you obtained with weighted regression in (iii).
////////////////////////////////////////////////////////////////////////////////

//load the data
use Framingham_HDL.dta,clear 

//set a seed using a random number:
set seed 1810

//generate shuffle:
gen u = runiform()
sort anychd u

//generate a variable to specify ppl selected in second-stage sample:
gen select = 0

//selects all (777) anychd==1 and 777 anychd==0:
by anychd: replace select = 1 if _n <= 777

//for sujects not in the second-stage sample, set their HDL and LDL to missing:
replace ldlc_cat = . if select == 0 
replace hdlc_cat2 = . if select == 0

//run logistic regression model using *"meanscor"* command:
xi: meanscor anychd i.ldlc_cat i.hdlc_cat2 i.sex i.prevhyp, first(sex prevhyp) second(ldlc_cat, hdlc_cat2)

//this gives output that is similar as the weighted logistic regression in (iii).


////////////////////////////////////////////////////////////////////////////////
//                               Question 2    
//
//-2. This exercise uses some of the data collected in a cross-sectional survey 
//    of schoolchildren in Stockholm schools to determine the risk of H. Pylori 
//    infection in children from infected family members. For all study 
//    children, information was gathered on household socioeconomic status (SES), 
//    and for immigrant children, whether the family’s country of origin had a high 
//    or low H. pylori prevalence. The information on H. Pylori infection in 
//    family members was only collected for a sub-sample of children. The 
//    family members of all infected children were tested, but in an effort to 
//    target the control families considered most informative, the intensity of 
//    sampling was different in the strata defined by SES (high vs. Low) and 
//    the H. Pylori prevalence (high vs. Low) in the child’s 
//    country of origin. Thus the second-stage sampling is stratified, with 
//    6 strata defined by case/control status, SES (high/low) and 
//    prevalence (high/low).
//
//    The data set provided, Hpylori_extract.dta, is an extract from the full 
//    data, containing only children from low prevalence countries, with the 
//    following information: H. Pylori (HP) status of the child (hpproband), HP 
//    status of the mother (hpmother), SES category (sescat) and an indicator 
//    variable (stage2) for whether the mother’s HP infection status is known.
//
//    Use a two-stage analysis to estimate the effect of a HP-positive mother 
//    on the child’s risk, adjusted for SES. (Note that since we are 
//    investigating only children from low-prevalence countries, the 
//    second-stage data is in 4 strata defined by case/control and high/low SES). 
//    Compare your results to those from the more detailed model in the Table 
//    below, where the full data set was analysed using a weighted regression 
//    (the coloumn “complete” refers to a simple logistic regression of the 
//    complete (i.e. stage 2) data, i.e. only children whose family members 
//    were tested.
//  
//    [see exercise for the table]
////////////////////////////////////////////////////////////////////////////////

//load the data
use Hpylori_extract.dta, clear

//create strata acoording to *hpproband* and *sescat*:
egen strat = group(hpproband sescat)

//count number of ppl in each created stratum:
bysort strat: gen stratsize = _N

//in each stratum, count number of ppl whose chlam data is available:
bysort strat: egen stratselected = total(stage2==1)

//calculate weights for the second-stage individuals:
gen weight = stratsize / stratselected

//Weighted logistic regression on second-stage sample:
logistic hpproband hpmother sescat [pw = weight] if stage2 == 1, vce(robust)

//Logistic regression using *meanscor*:
meanscor hpproband hpmother sescat, first(sescat) second(hpmother)

//The estimates are on the similar direction with comparable findings but SES is significant in the exericse not in the table.


////////////////////////////////////////////////////////////////////////////////
//                               Question 3(i)    
//
//-(i) Repeat the analysis in Question 1 using a two-stage case-control study 
//     with 777 randomly sampled men and 777 randomly sampled women selected at 
//     the second stage. 
////////////////////////////////////////////////////////////////////////////////

//load data
use Framingham_HDL.dta, clear

//set a seed using a random number:
set seed 1810

//generate shuffle:
gen u = runiform()
sort sex u

//generate a variable to specify ppl selected in second-stage sample:
gen select = 0

//selects 777 sex==1 and 777 sex==0:
by sex: replace select = 1 if _n <= 777

//check the results of selection: 
tabulate select sex

//for sujects not in the second-stage sample, set their HDL and LDL to missing:
replace ldlc_cat = . if select == 0 
replace hdlc_cat2 = . if select == 0

//calculate weights for cases and controls:
tabulate select anychd
gen weight = 777/416 if anychd == 1 & select == 1
replace weight = 2249/1138 if anychd == 0 & select == 1

//Weighted logistic regression on second-stage sample:
logistic anychd i.ldlc_cat i.hdlc_cat2 i.sex i.prevhyp [pw = weight] if select == 1, vce(robust)	

//Logistic regression using *meanscor*:
xi: meanscor anychd i.ldlc_cat i.hdlc_cat2 i.sex i.prevhyp, first(sex prevhyp) second(ldlc_cat, hdlc_cat2)	
	

////////////////////////////////////////////////////////////////////////////////
//                               Question 3(ii)    
//
//-(ii) Repeat the two-stage sampling experiment again but this time taking a 
//      second-stage sample of 1554 (i.e. 2 X 777) that is “balanced” across 
//      the eight strata defined by the anychd, sex and prevhyp (i.e. equal 
//      numbers in each stratum) using the following two approaches:
//      a. Select one-eight of the 1554 (195) from each stratum (or all, if a 
//         stratum has fewer)
//      b. Strategy a. will result in fewer than 1554. Select the “surplus” from 
//         the strata larger than 195 to yield a sample of size 1554 that is as 
//         “balanced” as possible
//
//      Re-run the weighted logistic regression for second-stage data for these 
//      two designs, using the appropriate weights. Compare your estimates to 
//      those in Question 1. 
//
//      Sampling strategy b. is just one way of allocating “surplus” records to 
//      achieve a given total sample size. However, these could have been chosen 
//      from any stratum/strata, provided they are chosen randomly: explain how 
//      this would impact on the analysis and suggest why an investigator might 
//      wish to take an “unbalanced” sample.
////////////////////////////////////////////////////////////////////////////////

//**Approach a.** 
//load data
use Framingham_HDL.dta, clear

//create strata acoording to *anychd, sex, and prevhyp*: 
egen strat = group(anychd sex prevhyp)
codebook strat

//we can see 8 strata are generated.
set seed 1810
gen u = runiform()
sort strat u
gen select = 0
by strat: replace select = 1 if _n <= 195
	
tabulate strat select
	
//The actual sample selected is smaller than expected, because some strata have less than 195 subjects.

//for sujects not in the second-stage sample, set their HDL and LDL to missing:
replace ldlc_cat = . if select == 0 
replace hdlc_cat2 = . if select == 0

//compute weight
by strat: gen stratsize = _N
by strat: egen stratselected = total(select == 1)
gen weight = stratsize / stratselected if select == 1

//Weighted logistic regression on second-stage sample:
logistic anychd i.ldlc_cat i.hdlc_cat2 i.sex i.prevhyp [pw = weight] if select == 1, vce(robust)


//**Approach b (for approximately 1554 sampled data).**
//load data
use Framingham_HDL.dta, clear

//create strata acoording to *anychd, sex, and prevhyp*: 
egen strat = group(anychd sex prevhyp)
codebook strat

//we can see 8 strata are generated.
set seed 1810
gen u = runiform()
sort strat u
gen select = 0
by strat: replace select = 1 if _n <= 195
	
tabulate strat select

//The actual sample selected (n=1371) is smaller than expected, because there are 2 strata have less than 195 subjects. To achieve the actual sample size of 1554, we still need 1554-1371=183 people to be equally selected from the other 6 strata which are larger than 195 (i.e., 183/6≈31 per stratum).

//save ppl who have been selected so far in a separate dataset:
preserve
keep if select==1
save 1.dta, replace
restore

//select the “surplus” from the strata larger than 195:
keep if select==0
sort strat u 
by strat: replace select = 1 if _n <= 31 & select==0

tabulate strat select

//append all the selected ppl:
append using 1.dta
tabulate strat select
save select.dta, replace

//for sujects not in the second-stage sample, set their HDL and LDL to missing:
replace ldlc_cat = . if select == 0 
replace hdlc_cat2 = . if select == 0

//compute weights:
sort strat
by strat: gen stratsize = _N
by strat: egen stratselected = total(select == 1)
gen weight = stratsize / stratselected if select == 1

//Weighted logistic regression on second-stage sample:
logistic anychd i.ldlc_cat i.hdlc_cat2 i.sex i.prevhyp [pw = weight] if select == 1, vce(robust)

//Compared to Question 1, the results are generally comparable. 

//Approach a. and b. will use different weights. An investigator might wish to take an “unbalanced” sample because investigator may be interested in certain group of people (specific strata). For example, man with history of heart disease.


//**Approach b (alternative when exactly 1554 sampled data).**
//load data
use Framingham_HDL.dta, clear

//create strata acoording to *anychd, sex, and prevhyp*: 
egen strat = group(anychd sex prevhyp)
codebook strat

//Given that there are 183 surplus and 183/6=30.5 per stratum among the 6 strata with >195 observations. Since the smallest among the 6 can sample 31 observations, we let the larger 5 stratum to sample 195+31=226 observations and the smallest to sample 183-31*5=28 observations (i.e., 195+28=223 observations).

//we can see 8 strata are generated.
set seed 1810
gen u = runiform()
sort strat u
gen select = 0
by strat: gen id = _n

//sample 226 for the larger 5 strata. Among the 3 smaller strata they will sample 81, 120 and 223 observations
replace select = 1 if strat==1 & id <= 226
replace select = 1 if strat==2 & id <= 226
replace select = 1 if strat==3 & id <= 226
replace select = 1 if strat==4 & id <= 226
replace select = 1 if strat==5 & id <= 120
replace select = 1 if strat==6 & id <= 226
replace select = 1 if strat==7 & id <= 81
replace select = 1 if strat==8 & id <= 223

tabulate strat select

//compute weights:
sort strat
by strat: gen stratsize = _N
by strat: egen stratselected = total(select == 1)
gen weight = stratsize / stratselected if select == 1

//Weighted logistic regression on second-stage sample:
logistic anychd i.ldlc_cat i.hdlc_cat2 i.sex i.prevhyp [pw = weight] if select == 1, vce(robust)
